/*
 * Software for Interactive Project Pool Game
 * Columbia University. New York, 2008
 * Authors:
 *   Abdulhamid Ghandour
 *   Thomas John
 *   Jaime Peretzman
 *   Bharadwaj Vellore
 * 
 * Desc:
 */
#include <stdio.h>
#include <unistd.h>

#include "system.h"
#include "io.h"
#include "fixedpoint.h"
#include "ui.h"
#include "calibration.h"

long long tableStartX;
long long tableEndX;
long long tableStartY;
long long tableEndY;

long long camStartX;
long long camEndX;
long long camStartY;
long long camEndY;

void doCalibration(){
  int calibrated = 0;
  unsigned long repos, last_repos;
  unsigned long counter = 0;
  long camStart = 0xFFFF, camEnd = 0xFFFF;
  long blackMarginSize;
 
  SET_GREEN_ROW_THR;
  SET_GREEN_COL_THR;

  blackMarginSize = BLACK_MARGIN_CONFIG;
  BLACK_MARGIN_SET(blackMarginSize);
  
  tableStartX = TABLE_START_HOZ_POS;
  tableEndX = TABLE_END_HOZ_POS;
  tableStartY = TABLE_START_VER_POS;
  tableEndY = TABLE_END_VER_POS;

  SET_CAMERA_SANDBOX_START_X(0);
  SET_CAMERA_SANDBOX_END_X(640);
  SET_CAMERA_SANDBOX_START_Y(0);
  SET_CAMERA_SANDBOX_END_Y(1024);
  
  IOWR_POS(VGA_BASE, 16,tableStartX);
  IOWR_POS(VGA_BASE, 18,tableEndX );
  IOWR_POS(VGA_BASE, 17, tableStartY);
  IOWR_POS(VGA_BASE, 19, tableEndY);
  IOWR_POS(VGA_BASE, 21, 1);

  LCD_Init();
  LCD_Show_Text("Calibrating..");
  
  usleep(200000);

  START_CALIBRATION;
  
  /*
   * Check reposition register and direct user to move camera
   * We use a counter here to make sure we check several times
   * to confirm that calibration is indeed complete and the
   * state is steady.
   */
   while(!calibrated){
     repos = READ_REPOS_REG;
     if(repos == 0){
       counter++;
       if(counter < 20){
         usleep(200000);
         printf("Calibrating. Please wait....\n");
         LCD_Init();
         LCD_Show_Text("Calibrating");
         LCD_Line2();
         LCD_Show_Text("Please wait...");
         continue;
       }else{
         calibrated = 1;
       }
       last_repos = 0;
     }else{
        counter = 0;
        /*
         * Find out the type of error and display message here
         */
        if(repos != last_repos){          
          printf("Repos = 0x%x\n",(unsigned int)repos);

          LCD_Init();
          LCD_Show_Text("Move Camera ");
          LCD_Line2();
        
          if((repos & 1) > 0 ){
            printf("Right ");  
            LCD_Show_Text("Right "); 
          }
          if((repos & 2) > 0){
            printf("Left ");   
            LCD_Show_Text("Left ");
          }
          if((repos & 4) > 0){
            printf("Down or back ");
            LCD_Show_Text("Down or Back");
          }
          if((repos & 8) > 0){
            printf("Up or back ");   
            LCD_Show_Text("Up or Back");
          }
          if((repos & 16) > 0){
            printf("Backwards ");   
            LCD_Show_Text("Backwards ");
          }
          if((repos & 32) > 0){
            printf("Forward ");   
            LCD_Show_Text("Forward ");
          }
          if(repos == 64){
            printf("Point camera at Table\n");
            LCD_Show_Text("Point At Table ");
          }
          printf("\n");
          last_repos = repos;
        }
        continue;
     }
   }
   printf("Done Calibration\n");
   camStart = READ_CAMERA_START;
   camEnd = READ_CAMERA_END;

   camStartX = (long long)(camStart & 0x0007FF);
   camStartY = (long long)((camStart & 0x3FF800) >> 11);
   camEndX = (long long)(camEnd & 0x0007FF);
   camEndY = (long long)((camEnd & 0x3FF800) >> 11);

   SET_CAMERA_SANDBOX_START_X((long)camStartX + CAMERA_CROP_MARGIN_HOZ);
   SET_CAMERA_SANDBOX_END_X((long)camEndX - CAMERA_CROP_MARGIN_HOZ);
   SET_CAMERA_SANDBOX_START_Y((long)camStartY + CAMERA_CROP_MARGIN_VER);
   SET_CAMERA_SANDBOX_END_Y((long)camEndY - CAMERA_CROP_MARGIN_VER);
   
   printf("Cam Start X = %ld, End X = %ld\n", (long)camStartX, (long)camEndX);
   printf("Cam Start Y = %ld, End Y = %ld\n", (long)camStartY, (long)camEndY);

   STOP_CALIBRATION;
}

void wait_fn(){
  int i=0;
  int j=0;
  for (;i<=4001;){
    i++;
  }
  while((IORD_16DIRECT(VGA_BASE, 12*2) & 0x0001) ==1);
  for(;j<=500;){
    j++;
  }
}
    
void wait_fn2(){
  int i=0;
  for (;i<=150001;){
    i++;
  }
  while((IORD_16DIRECT(VGA_BASE, 20*2) & 0x0001) ==1);
}
    
/*void wait_fn2(){
  int i=0;
  int j=0;
  for (;i<=50001;){
    i++;
  }
  while((IORD_16DIRECT(VGA_BASE, 20*2) & 0x0001) ==1);
  for(;j<=500;){
    j++;
  }
}*/
    
void wait_fn3(){
  int i=0;
  for (;i<=5000001;){
    i++;
  }
}
         
void wait_fn4(){
  int i=0;
  for (;i<=4001;){
    i++;
  }
}

void wait_fn5(){
  int i=0;
  for(;i<=600001;){
    i++;
  }
}

int calibrate(){
  int cross_H = 150;
  int cross_V = 150;
  int x1 = 0;
  int x2 = 320;
  int y1 = 140;
  int y2 = 180;
  int temp_x1=0, temp_x2=0, temp_y1=0,temp_y2=0;
  int cal_flag=0;
  int read_in=0;
  int delta_stick = 4; // Defines the stepsize in the calibration sticks
  int border_margin = 15;

  int xin, yin;

  while (1) {
    wait_fn2();
   //printf("H = %d, V = %d\n",cross_H,cross_V);
    IOWR_POS(VGA_BASE, 16, x1);
    IOWR_POS(VGA_BASE, 18, x2);
    IOWR_POS(VGA_BASE, 17, y1);
    IOWR_POS(VGA_BASE, 19, y2);
    IOWR_POS(VGA_BASE, 21, 0);
    IOWR_POS(VGA_BASE, 13, cross_H);
    IOWR_POS(VGA_BASE, 14, cross_V);
    wait_fn4 ();
    xin = IORD_32DIRECT(VISION_BASE, 0);
    yin = (xin >> 16) & 0x00007FFF;
    xin = xin & 0x0000FFFF;
    
    cross_H = xin;
    cross_V = yin;
    
    read_in+=1;
    wait_fn2();
  
    if (cal_flag==0&& read_in>2)
    x2 = x2 - delta_stick;
    
    if(NO_CUE_DETECTED && cal_flag==0 && read_in>2)
    {   
        wait_fn3();
        temp_x1 = x2 + delta_stick;
        x2=639;
        x1=320;
        y1=140;
        y2=180;
        cal_flag=1;
        read_in=0;
    }
    if (cal_flag==1 && read_in>2)
    x1 = x1 + delta_stick;
    
    if(NO_CUE_DETECTED  && cal_flag==1 && read_in>2)
    {    
         wait_fn3();
         temp_x2 = x1 - delta_stick;
         x1= 320;
         x2= 380;
         y1=0;
         y2=240;
         cal_flag=2;
         read_in=0;    
    }
    if (cal_flag==2 && read_in>2)
    y2 = y2 - delta_stick;
    
    if(NO_CUE_DETECTED  && cal_flag==2 && read_in>2)
    {   
        wait_fn3();
        temp_y1 = y2 + delta_stick;
        x1=120;
        x2=180;
        y1=240;
        y2=480;
        cal_flag=3;
        read_in=0;
    }
    if (cal_flag==3 && read_in>2)
    {
      y1 = y1 + delta_stick;
    }
    
    if(NO_CUE_DETECTED  && cal_flag==3 && read_in>2)
    {
      temp_y2 = y1 - delta_stick;
      tableStartX = (long long)temp_x1 + border_margin;
      tableEndX = (long long)temp_x2 - border_margin;
      tableStartY = (long long)temp_y1 + border_margin;
      tableEndY = (long long)temp_y2 - border_margin;
      IOWR_POS(VGA_BASE, 16,tableStartX);
      IOWR_POS(VGA_BASE, 18,tableEndX );
      IOWR_POS(VGA_BASE, 17, tableStartY);
      IOWR_POS(VGA_BASE, 19, tableEndY);
      IOWR_POS(VGA_BASE, 21, 1);
      break;
    }
  }
  return 0;
}


