--
-- DE2 (Cyclone-II) Entity for Interactive Project Game
-- Authors:
--      Abdulhamid Ghandour
--      Thomas John
--      Jaime Peretzman
--      Bharadwaj Vellore
--
-- Desc:
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity uicontroller is
  
  port (
    clk        : in  std_logic;
    reset_n    : in  std_logic;
    read       : in  std_logic;
    write      : in  std_logic;
    chipselect : in  std_logic;
    address    : in  unsigned(4 downto 0);
    readdata   : out unsigned(31 downto 0);
    writedata  : in  unsigned(31 downto 0);
    hex0       : out std_logic_vector(7 downto 0);
    hex1       : out std_logic_vector(7 downto 0);
    hex2       : out std_logic_vector(7 downto 0);
    hex3       : out std_logic_vector(7 downto 0);
    hex4       : out std_logic_vector(7 downto 0);
    hex5       : out std_logic_vector(7 downto 0);
    hex6       : out std_logic_vector(7 downto 0);
    hex7       : out std_logic_vector(7 downto 0);
    key        : in std_logic_vector(3 downto 0);
    switch     : in unsigned (17 downto 0)
  );
end uicontroller;

architecture rtl of uicontroller is

  type ram_type is array(31 downto 0) of unsigned(31 downto 0);
  signal RAM : ram_type;
  signal ram_address : unsigned(4 downto 0);
  signal int_key1 : std_logic;
  signal int_key2 : std_logic;
begin
  ram_address <= address;

  reg_loader: process(clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then
        RAM(0) <= (others => '1');
        RAM(1) <= (others => '1');
        RAM(2) <= (others => '1');
        RAM(3) <= (others => '1');
        RAM(4) <= (others => '1');
        RAM(5) <= (others => '1');
        RAM(6) <= (others => '1');
        RAM(7) <= (others => '1');
      else
        if chipselect = '1' then
          if write = '1' then
            RAM(to_integer(ram_address)) <= writedata;
          elsif read = '1' then
            readdata <= RAM(to_integer(ram_address));
          end if;
        else
 		  if RAM(8)(0) = '1' then
            RAM(8)(0) <= int_key1;
          end if;
          if RAM(9)(0) = '1' then
            RAM(9)(0) <= int_key2;
          end if;
		  RAM(10)(17 downto 0) <= switch(17 downto 0);
        end if;
      end if;
    end if;
  end process reg_loader;

  seven_segment_driver: process(clk)
    begin
    if rising_edge(clk) then
      if reset_n = '0' then
        -- do nothing
      else
        hex0(0) <= RAM(0)(0);
        hex0(1) <= RAM(0)(1);
        hex0(2) <= RAM(0)(2);
        hex0(3) <= RAM(0)(3);
        hex0(4) <= RAM(0)(4);
        hex0(5) <= RAM(0)(5);
        hex0(6) <= RAM(0)(6);
        hex0(7) <= RAM(0)(7);
        
        hex1(0) <= RAM(1)(0);
        hex1(1) <= RAM(1)(1);
        hex1(2) <= RAM(1)(2);
        hex1(3) <= RAM(1)(3);
        hex1(4) <= RAM(1)(4);
        hex1(5) <= RAM(1)(5);
        hex1(6) <= RAM(1)(6);
        hex1(7) <= RAM(1)(7);

        hex2(0) <= RAM(2)(0);
        hex2(1) <= RAM(2)(1);
        hex2(2) <= RAM(2)(2);
        hex2(3) <= RAM(2)(3);
        hex2(4) <= RAM(2)(4);
        hex2(5) <= RAM(2)(5);
        hex2(6) <= RAM(2)(6);
        hex2(7) <= RAM(2)(7);
        
        hex3(0) <= RAM(3)(0);
        hex3(1) <= RAM(3)(1);
        hex3(2) <= RAM(3)(2);
        hex3(3) <= RAM(3)(3);
        hex3(4) <= RAM(3)(4);
        hex3(5) <= RAM(3)(5);
        hex3(6) <= RAM(3)(6);
        hex3(7) <= RAM(3)(7);     

        hex4(0) <= RAM(4)(0);
        hex4(1) <= RAM(4)(1);
        hex4(2) <= RAM(4)(2);
        hex4(3) <= RAM(4)(3);
        hex4(4) <= RAM(4)(4);
        hex4(5) <= RAM(4)(5);
        hex4(6) <= RAM(4)(6);
        hex4(7) <= RAM(4)(7);
        
        hex5(0) <= RAM(5)(0);
        hex5(1) <= RAM(5)(1);
        hex5(2) <= RAM(5)(2);
        hex5(3) <= RAM(5)(3);
        hex5(4) <= RAM(5)(4);
        hex5(5) <= RAM(5)(5);
        hex5(6) <= RAM(5)(6);
        hex5(7) <= RAM(5)(7);

        hex6(0) <= RAM(6)(0);
        hex6(1) <= RAM(6)(1);
        hex6(2) <= RAM(6)(2);
        hex6(3) <= RAM(6)(3);
        hex6(4) <= RAM(6)(4);
        hex6(5) <= RAM(6)(5);
        hex6(6) <= RAM(6)(6);
        hex6(7) <= RAM(6)(7);
        
        hex7(0) <= RAM(7)(0);
        hex7(1) <= RAM(7)(1);
        hex7(2) <= RAM(7)(2);
        hex7(3) <= RAM(7)(3);
        hex7(4) <= RAM(7)(4);
        hex7(5) <= RAM(7)(5);
        hex7(6) <= RAM(7)(6);
        hex7(7) <= RAM(7)(7);

        int_key1 <= key(1); -- key(0) is not captured.
        int_key2 <= key(2);
      end if;
    end if;
  end process seven_segment_driver;
end rtl;
